function [ Vol, img_info] = load_img_stack(folder,doParallel,varargin)

%% Optional input arguments

% only want 1 optional input at most
numvarargs = length(varargin);
if numvarargs > 5
    error( 'Too many optional variables. This function only accepts 2: row_range, col_range');
end

% set defaults for optional inputs
optargs = {1 [] [] []}; %default value for optional arguments

% overwrite with the ones specified in varargin.
optargs(1:numvarargs) = varargin;

% Place optional args in memorable variable names
[ imgRszScale,row_range,col_range,slice_range ] = optargs{:};


%% load prerequisites
if (doParallel)
    [pathStr, ~ ,~] = fileparts(mfilename('fullpath'));
    parforProgress_directory = [ pathStr filesep 'parfor_progress'];
    addpath(parforProgress_directory);  
end

%% do a quick scan to find number of images in folder
[ img_file_names, img_info] = parseFolder(folder);

numZSlices = length(img_file_names);

%% Transfer all (or parts) of the image into the volume

if (~isempty(row_range) ) || (~isempty(col_range) ) || (~isempty(slice_range))
    
    if strcmpi(row_range,'max')
        img_row_cords = 1:img_info.dims(1);
    else
        img_row_cords = row_range;
    end
    
    if strcmpi(col_range,'max')
        img_col_cords = 1:img_info.dims(2);
    else
        img_col_cords = col_range;
    end
    
    if strcmpi(slice_range,'slice_range')
        img_slices  = numZSlices;
    else
        img_slices  = slice_range;
    end
else

    img_row_cords = 1:img_info.dims(1);
    img_col_cords = 1:img_info.dims(2);
    img_slices = 1:numZSlices;
end

if (imgRszScale == 1) %calculate how big a resized region will be    
    rowSize = length(img_row_cords);
    colSize = length(img_col_cords);    
else
    testimg = ones(length(img_row_cords),length(img_col_cords));
    [rowSize,colSize] = size(imresize(testimg,imgRszScale,'bicubic'));     
end
 if strcmpi(img_info.class,'logical')
    Vol = false(rowSize,colSize,length(img_slices));
else
    Vol = zeros(rowSize,colSize,length(img_slices),img_info.class);
end

%open a matlab pool to speed up loading (useful when there is postprocessing involved
if (doParallel)
    if (isempty(gcp('nocreate')))        
       parpool('local'); 
    end    
end

%do the image loading
if (doParallel)
    
    parfor_progress(length(img_slices));
    parfor i = 1:length(img_slices)        

        %read in an image slice
        img_file_path = [folder filesep img_file_names{img_slices(i)}];
        [ img ] = parseImage(img_file_path,img_info,i);

        %Additional filtering
        %img = medfilt2(img,[5 5]);                           
        img_region = img(img_row_cords,img_col_cords);

        if (imgRszScale == 1)           
            Vol(:,:,i) = img_region;
        else %GPU based image resizing
            %img_region_GPU = gpuArray(img_region);
            %img_regionRsz_GPU = imresize(img_region_GPU,imgRszScale,'bicubic');
            %Vol(:,:,i) = gather(img_regionRsz_GPU);        
            Vol(:,:,i) = imresize(img_region,imgRszScale,'bicubic');
        end
        
        
        parfor_progress;
        
    end

    parfor_progress(0);
%     delete(gcp('nocreate'));

else
    [defProgState] = textprogressbar('Loading images: ',0);    
    for i = 1:length(img_slices)
        
        %read in an image slice
        img_file_path = [folder filesep img_file_names{img_slices(i)}];
        [ img ] = parseImage(img_file_path,img_info,i);
        
        %Additional filtering
        %img = medfilt2(img,[5 5]);
        img_region = img(img_row_cords,img_col_cords);
        
        if (imgRszScale == 1)
            Vol(:,:,i) = img_region;
        else %GPU based image resizing
            %img_region_GPU = gpuArray(img_region);
            %img_regionRsz_GPU = imresize(img_region_GPU,imgRszScale,'bicubic');
            %Vol(:,:,i) = gather(img_regionRsz_GPU);
            
            Vol(:,:,i) = imresize(img_region,imgRszScale,'bicubic');
        end
        
        [defProgState] = textprogressbar(round((i/length(img_slices))*100),defProgState);
    end
    textprogressbar('done',-2);
end

end




